
module Mini.Lexer (
    lexer,
    ) where

import GLL.Combinators (Token(..))
import Data.Char (isSpace, isDigit, isAlpha)
import Text.Regex.Applicative

lexer :: String -> [Token]
lexer [] = []
lexer s = 
    let re =    (Just <$> lTokens)
            <|> (Nothing <$ some (psym isSpace))
            <|> (Nothing <$ sym '#' <* many (psym ((/=) '\n')))
    in case findLongestPrefix re s of
        Just (Just tok, rest)   -> tok : lexer rest
        Just (Nothing,rest)     -> lexer rest
        Nothing                 -> error ("lexical error on\n " ++ s)

lTokens :: RE Char Token
lTokens = 
        lKeyword "var"
    <|> lKeyword "print"
    <|> lKeyword "throw"
    <|> lKeyword "try"
    <|> lKeyword "catch"
    <|> lKeyword "if"
    <|> lKeyword "then"
    <|> lKeyword "else"
    <|> lKeyword "while"
    <|> lKeyword "begin"
    <|> lKeyword "end"
    <|> lKeyword "for"
    <|> lKeyword "procedure"
    <|> lKeyword "return"
    <|> lKeyword "true"
    <|> lKeyword "false"
    <|> lKeyword "+"
    <|> lKeyword "*"
    <|> lKeyword "-"
    <|> lKeyword "/"
    <|> lKeyword "<"
    <|> lKeyword "<="
    <|> lKeyword ">="
    <|> lKeyword ">"
    <|> lKeyword "=="
    <|> lKeyword "!="
    <|> lKeyword ":="
    <|> lKeyword "!"
    <|> lKeyword "^"
    <|> lKeyword "not"
    <|> lKeyword "null"
    <|> lKeyword "lift"
    <|> lKeyword "eval"
    <|> lKeyword "[|"
    <|> lKeyword "[e|"
    <|> lKeyword "[c|"
    <|> lKeyword "[d|"
    <|> lKeyword "|]"
    <|> lCharacters 
    <|> IntLit . Just . read    <$> some (psym isDigit)
    <|> StringLit . Just        <$  sym '"' <*> many (psym ((/=) '"')) <* sym '"'
    <|> IDLit  . Just           <$> some (psym (\c -> isAlpha c || c == '_')) 

lCharacters = foldr (\c a -> Char c <$ sym c <|> a) empty "=();,![]{}^$"
lKeyword k  = Keyword k <$ string k
