
%if false
\begin{code}
import GLL.Combinators hiding (BoolLit, IntLit, StringLit, parse, lexer)

import Funcons.EDSL hiding (Vector)
import Funcons.Core.Manual hiding (bool_)
import Funcons.MetaProgramming as MP
import Funcons.Core hiding (string_,bool_) 
import qualified Funcons.Core as Core
import Funcons.Tools
import Funcons.Mini.Mini as Mini

import IML.Trans.FromFuncons (mkQueryFromFCT)
import IML.Printer (showQuery, showResetBindings)

import Mini.Lexer

import Control.Monad
import System.Environment

import qualified Data.Map as M
\end{code}
%endif

\authoredby{C}

\lstdefinestyle{mini}{showstringspaces=false,numbers=left,stepnumber=1,language=bash,frame=single,deletekeywords={if,throws,else},morekeywords={catch,try,true,false,var,then,print,begin,end,if,else,procedure,null}}
\lstset{style=mini}

\Mini{} is a basic procedural programming language with arrays, exceptions, recursive procedures, and a check for uninitialised variables.
%
The following program computes the first $n$ Fibonacci numbers with dynamic programming and highlights most of the features of \Mini{}:
%
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/fib-dyn.mini}
\label{code:fib_dyn_mini}
\end{cofloat}%

%
\Mini{} has several program constructs, categorised into expressions, commands, and declarations.
%
In the next sections we explain each construct informally and define the syntax and semantics of each construct formally with the tools discussed in Chapter~\ref{chap:eval_preliminaries}.
%
We begin by listing all constructs, giving the symbol expressions that capture the syntax of expressions, commands, and declarations.
%
\begin{cofloat}%
\begin{code}
syn_expr = "expressions"    <:=   syn_ops                 -- prefix and infix operators 
                            <||>  syn_parens              -- grouped expressions  
                            <||>  syn_literals            -- literal values
                            <||>  syn_ident               -- identifiers
                            <||>  syn_proc                -- procedure invocation   
                            <||>  syn_null                -- testing variable initialisation
                            <||>  syn_array               -- array construction
                            <||>  syn_array_idx           -- array indexing

syn_command = "commands"    <:=>  syn_cmdexpr             -- expressions as commands
                            <||>  syn_print               -- printing output
                            <||>  syn_assign              -- variable assignment 
                            <||>  syn_block_locals        -- code blocks
                            <||>  syn_ite                 -- if-then-else
                            <||>  syn_while               -- while-loops
                            <||>  syn_for                 -- for-loops
                            <||>  syn_throw               -- throwing exceptions
                            <||>  syn_trycatch            -- catching exceptions
                            <||>  syn_return              -- returning values

syn_decl = "declarations"   <:=>  syn_vardecl             -- variable declarations
                            <||>  syn_procdecl            -- procedure declarations
\end{code}%
\end{cofloat}%
%



\section{Basic Expressions}

\label{sec:mini_basic_exprs}%

\Mini{} expressions evaluate to integers, booleans and strings.
%
Integers are written as a non-empty sequence of digits.
%
The sequence of characters \verb+-123+ is interpreted as the application of the prefix operator \lstinline+-+ to the integer \verb+123+.
%
Boolean values are written as either \lstinline-true- or \lstinline-false-.
%
Strings are any sequence of characters in between double quotes.
%
For a double quote to appear in a string, it has to be escaped using `\textbackslash'.
%
\begin{cofloat}%
\begin{code}
syn_literals = "literals"
  <:=>  int_         <$$>  int_lit          -- non-empty sequence of digits
  <||>  bool_ True   <$$   keyword "true"
  <||>  bool_ False  <$$   keyword "false"
  <||>  string_      <$$>  string_lit       -- characters within quotes
\end{code}%
\end{cofloat}%%

\paragraph{Operators}

Infix operators have two operands evaluated in left to right order.
%
Prefix operators have a single operand.
%
\begin{cofloat}%
\begin{code}
sem_infix e1 op e2   = apply_ [apply_ [bound_built_in op, e1], e2]
sem_prefix op e1     = apply_ [bound_built_in op, e1]

bound_built_in op    = bound_ [string_append_ [string_ "built-in ", string_ op]]
\end{code}%
\end{cofloat}%
%
\indent
Operators are built-in procedures: at runtime a binding is active that binds a unique identifier (e.g. |"built-in +"| for $+$) to the body of a procedure.
%
The procedure receives the values obtained by evaluating the operands and returns a result value. 
%
The precise bindings that are active for each operator are given in Figure~\ref{fig:operators}.

\begin{figure}[t]
\begin{code}
operator_semantics =
        [   ( "built-in +",       infix_op [integer_add_ [given1, given2]] )
        ,   ( "built-in -",       infix_op [integer_subtract_ [given1, given2]] )
        ,   ( "built-in minus",   prefix_op [integer_subtract_ [int_ 0, given_]])
        ,   ( "built-in *",       infix_op [integer_multiply_ [given1, given2]] )
        ,   ( "built-in /",       infix_op [
                if_true_else_ [is_equal_ [int_ 0, given2]
                              ,throw_ [string_ "division-by-zero"]
                              ,integer_divide_ [given1, given2]] ]
            )
        ,   ( "built-in not" ,   prefix_op [not_ [given_]] )
        ,   ( "built-in !"   ,   prefix_op [not_ [given_]] )
        ,   ( "built-in ==",     infix_op [is_equal_ [given1, given2]] )
        ,   ( "built-in !=",     infix_op [not_ [is_equal_ [given1, given2]]] )
        ,   ( "built-in <",      infix_op [is_less_ [given1, given2]] )
        ,   ( "built-in <=",     infix_op [is_less_or_equal_ [given1, given2]] )
        ,   ( "built-in >",      infix_op [is_greater_ [given1, given2]] )
        ,   ( "built-in >=",     infix_op [is_greater_or_equal_ [given1, given2]] )
        ,   ( "built-in ^",      infix_op [string_append_ [given1, given2]])
        ]

infix_op body   = curry_ [prefix_op body]
prefix_op body  = function_ [abstraction_ body]

given1          = first_ [tuple_elements_ [given_]]
given2          = second_ [tuple_elements_ [given_]]
\end{code}
\caption{\Mini{}'s built-in procedures and their implementations as \cbsFC{functions}.}
\label{fig:operators}
\end{figure}

An infix operator is written between two expressions and may be left-associative, right-associative or both.  
%
A prefix operator is written before an expression. 

%
Operators have a relative precedence, determining which operator applications are evaluated first.
%
The fixity, precedence, and associativity of each operator are listed in the operator table given Figure~\ref{fig:precedence} (operator tables are discussed in~\ref{sec:operator_tables}).
%
\begin{figure}[th]
\begin{code}
operator_syntax = 
  [ (0,   [ ("not", Prefix  sem_prefix) ] )
  , (1,   [ ("==",  Infix   sem_infix   LAssoc)  , ("!=",  Infix   sem_infix   LAssoc) ] )
  , (2,   [ ("<",   Infix   sem_infix   LAssoc)  , ("<=",  Infix   sem_infix   LAssoc)
          , (">",   Infix   sem_infix   LAssoc)  , (">=",  Infix   sem_infix   LAssoc) ] )
  , (3,   [ ("+",   Infix   sem_infix   LAssoc)  , ("-",   Infix   sem_infix   LAssoc)
          , ("^",   Infix   sem_infix   LAssoc) ] )
  , (5,   [ ("*",   Infix   sem_infix   LAssoc)  , ("/",   Infix   sem_infix   LAssoc) ] )
  , (9,   [ ("!",   Prefix  sem_prefix)          , ("-",   Prefix  sem_minus) ] )
  ]

syn_ops = fromOpTable "operators" (opTableFromList operator_syntax) syn_expr_lit
sem_minus _ e1  = sem_prefix "minus" e1 
\end{code}%
\caption{The precedence, fixity and associativity of \Mini{}'s operators.}%
\label{fig:precedence}%
\end{figure}
%

Parentheses are used to group expressions.
%
\begin{cofloat}%
\begin{code}
syn_parens = parens syn_expr
\end{code}%
\end{cofloat}%
%
\section{Variables}
\label{sec:mini_variables}

% Names can be given to the results of expressions (values) in order to avoid performing the same computation multiple times. 
%
% Instead of writing the same expression in several locations, we give a name (identifier) to (the result of) the expressions once and reuse the name in each of those locations. 
%
Identifiers \emph{bind} variables or procedures. 
%
Variables have values \emph{assigned} to them. 
%
The same value can be assigned to multiple variables and multiple identifiers can bind the same variable. 
%
Bindings are temporary: they become active at a declaration site, and expire at the end of a block of code. 
%
Assignments do not expire (but a garbage collector may flush variables which are no longer bound). 
%
Bindings can be overwritten by binding the same identifier to a different variable or procedure. 
%
Assignments can be overwritten by assigning a different value to the same variable.
%
The details of these concepts are discussed in this section.
%
An identifier is a non-empty sequence of alphabetical characters and underscores. 

\paragraph{Variable declarations}


Variable declarations are executed to activate a binding from an identifier to a variable. 
%
A variable declaration is written by the keyword \textbf{var} followed by an identifier and a semicolon.
%
\begin{cofloat}%
\begin{code}
syn_vardecl =  "var-decl" <:=> 
                sem_vardecl <$$ keyword "var" <**> syn_id <** keychar ';'
syn_id = string_ <$$> id_lit -- alphabetical characters and underscores
\end{code}%
\end{cofloat}%
%
Executing a variable declaration involves creating a new, uninitialised variable as a side-effect. 
%
The main result of executing a declaration is the activation of a binding: the identifier is bound to the newly created variable.
%
\begin{cofloat}%
\begin{code}
sem_vardecl i = bind_ [i, allocate_variable_ [values_]]
\end{code}%
\end{cofloat}%
%
\paragraph{Expressions}
%
A \Mini{} expression that evaluates to a variable is referred to as an \emph{l-value} expression.
%
There are two forms of l-value expressions.
%
The first comprises a single identifier:
%
\begin{cofloat}%
\begin{code}
syn_ident   = sem_var <$$> syn_id 
sem_var id  = bound_ [id]
\end{code}%
\end{cofloat}%
%
The second type of l-value expressions is introduced in Section~\ref{sec:mini_arrays}.
%

If an l-value expression $e$ occurs in a context where a value is expected, the variable to which $e$ evaluates is dereferenced.
%
\begin{cofloat}%
\begin{code}
syn_expr_lit    = ensure_lit <$$> syn_expr
ensure_lit e1   = else_   [current_value_ [e1]
                          ,sem_throw "uninitialised-reference-component"]
\end{code}%
\end{cofloat}%
%
A \Mini{} expression that evaluates to a (non-variable) value is referred to as an \emph{r-value} expression.
%
For example, the expression \lstinline-x+y- is an r-value expression, and the subexpressions \lstinline-x- and \lstinline-y- are r-value expressions as well.
%

\Mini{} programs in which an identifier occurs in a place where it is not bound are not valid. 
%
If an expression is evaluated in which an identifier is bound to an uninitialised variable, then the |"uninitialised-reference-component"| exception (see Section~\ref{sec:mini_exceptions}) is raised.
%
%
Programs in which r-value expressions occur in a context where an l-value expression is expected are not valid.
%

A programmer can check whether a variable is initialised by writing $\textbf{null}\;e$, where $e$ is an l-value expression.
%
This Boolean expression evaluates to \lstinline-true- if $e$ evaluates to an uninitialised variable and \lstinline-false- otherwise.
%
\begin{cofloat}%
\begin{code}
syn_null     = sem_null <$$ keyword "null" <**> syn_expr
sem_null e1  = else_ [sequential_ [effect_ [assigned_ [e1]], bool_ False], bool_ True]
\end{code}%
\end{cofloat}%
%
For example, the following program prints \lstinline-true- twice:
%

\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/null-basic.mini}%
\end{cofloat}%%


\section{Basic Commands}

Commands are executed for their effects: printing values and mutating the assignment of simple or composite variables (see Section~\ref{sec:mini_arrays}).
%

The print command is written as the keyword \lstinline-print- followed by an r-value expression and a semicolon.
%
\begin{cofloat}%
\begin{code}
syn_print = sem_print <$$ keyword "print" <**> syn_expr_lit <** keychar ';'
\end{code}%
\end{cofloat}%
%
The expression is evaluated and its values is translated into a string. 
%
The string is extended with the newline character and printed to the standard output.
%
\begin{cofloat}%
\begin{code}
sem_print e1 = print_ [string_append_   [mini_show_ [e1] -- defined in Section~\ref{sec:mini_show}
                                        ,string_ "\n"]]
\end{code}%
\end{cofloat}%
%
\indent
An assignment is an l-value expression followed by the |=|-operator and an r-value expression followed by a semicolon.
%
\begin{cofloat}%
\begin{code}
sem_assign e1 e2  = assign_ [e1, e2]
syn_assign        =  "assignment" <:=> sem_assign <$$> 
                        syn_expr <** keychar '=' <**> syn_expr_lit <** keychar ';'
\end{code}%
\end{cofloat}%
%
For example, \lstinline-x = 3+y;- is a valid assignment, but \lstinline-x<0 = 3+y;- is not, because \lstinline-x<0- is not an l-value expression.
%

\Mini{} expressions may have side-effects when they involve procedure invocation (see Section~\ref{sec:procedures}). 
%
An expression can be evaluated for its effects only, by executing the expression as a command.
%
An r-value expression followed by a semicolon is a command.
%
\begin{cofloat}%
\begin{code}
sem_cmdexpr e1  = effect_ [e1]
syn_cmdexpr     = sem_cmdexpr <$$> syn_expr_lit <** keychar ';'
\end{code}%
\end{cofloat}%

\paragraph{Blocks of code}
\label{sec:mini_blocks}

A \Mini{} program (see Section~\ref{sec:mini_programs}) is a sequence of globals, where a global is a declaration or a command.
%
A sequence of globals is executed in order, so that the side-effects of each global can be witnessed by subsequent globals.
%
On top of that, a declaration makes a new binding available to subsequent globals.
%
\begin{cofloat}
\begin{code}
syn_globals   = "globals"   <:=>  foldr_multiple syn_global null_
syn_global :: SymbExpr Token (Funcons -> Funcons) 
syn_global    = "global"    <:=>  activate_in  <$$> syn_decl
                            <||>  and_then     <$$> syn_command

activate_in env fct   = scope_ [env,fct]
and_then cmnd fct     = sequential_ [cmnd,fct]
\end{code}
\end{cofloat}%
%
A code block is also a sequence of declarations and commands, delimited by braces or by the keywords \textbf{begin} and \textbf{end}.
%
\begin{cofloat}%
\begin{code}
syn_locals  = syn_globals
syn_local   = syn_global

syn_block_locals =  "block-locals"   <:=>  keyword "begin" **> syn_locals <** keyword "end" 
                                     <||>  braces syn_locals
\end{code}%
\end{cofloat}%
%
A code block may occur in place of a command, as in the following program:
%
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/decl_in_scope.mini}%
\end{cofloat}%
%
The lifetime of a variable ends at the end of the code block in which it is declared.
%
Therefore \lstinline-y- is not in scope after line 6 in the program above.
%
Identifier \lstinline-x- is in scope however, and line 7 prints \verb-true-, because assignments persist.
%

Code blocks can also appear as components of a control-flow command (Section~\ref{sec:mini_control_flow}).
%
In this case, a single command (without delimiters) can be written instead of a code block.
%
\begin{cofloat}%
\begin{code}
syn_branch = "component-block"  <:=   syn_block_locals   
                                <||>  (flip ($) null_) <$$> syn_local
\end{code}%
\end{cofloat}%
%
As an example, consider the following program with an \lstinline-else--branch formed out of a single command.
%
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/null-if.mini}%
\end{cofloat}%

\section{Procedures}
\label{sec:procedures}

A user-defined procedure is \emph{declared} once and can subsequently be \emph{invoked} zero or more times.
%
A procedure is declared by writing the keyword \textbf{procedure} followed by the procedure's name --- an identifier --- a comma-separated (possibly empty) sequence of variable declarations within parentheses, and the procedure's body --- a code block.
%
The variables mentioned within a procedure declaration are the procedure's formal parameters.
%
\begin{cofloat}
\begin{code}
syn_procdecl = sem_procdecl <$$
  keyword "procedure" <**> syn_id <**> parens syn_formals <**> syn_branch
syn_formals :: SymbExpr Token [Funcons]
syn_formals = "formals" <:=> multipleSepBy (keyword "var" **> syn_id) (keychar ',')
\end{code}
\end{cofloat}%
%
A procedure declaration binds the name of the procedure to its body.
%
When invoked, the formal parameters (if any) are bound to the arguments given to the procedure.
%
As shown later, the arguments are fresh, initialised variables.
%
\begin{cofloat}%
\begin{code}
sem_procdecl id params body = bind_recursively_ [id, abs]
  where   abs   | null params   = closure_ [body]
                | otherwise     = foldr combine body params
          combine param code =  
            function_ [closure_ [scope_ [bind_ [param, given_], code]]]

\end{code}%
\end{cofloat}%
%
A procedure may be invoked recursively, i.e. the body of a procedure may contain an invocation of the procedure itself.
%
A procedure invocation is an expression written as the procedure's name followed by the actual parameters, a comma-separated sequence of r-value expressions within parentheses.
%
\begin{cofloat}%
\begin{code}
syn_proc = sem_invoke <$$> syn_id <**> parens syn_actuals
syn_actuals :: SymbExpr Token [Funcons]
syn_actuals = "actuals" <:=> multipleSepBy syn_expr_lit (keychar ',')
\end{code}%
\end{cofloat}%
%
A procedure invocation evaluates to the value returned by the procedure.
%
The evaluation involves finding the procedure body bound to the identifier, and evaluating the actual parameters (if any).
%
The values that result are assigned to fresh variables, forming the arguments that are given to the procedure body.
%
\begin{cofloat}%
\begin{code}
sem_invoke id args = handle_return_ [invocation]
  where   invocation  | null args   = enact_ [bound_ [id]]
                      | otherwise   = foldl app (bound_ [id]) args
          app abs arg = apply_  [abs,allocate_initialised_variable_ [values_,arg]]
\end{code}%
\end{cofloat}%
%
A return command within a procedure body terminates procedure invocations, potentially before all declarations and commands in the body have been executed.
%
A return command is written as the keyword \textbf{return} followed by an r-value expression and a semicolon.
%
\begin{cofloat}%
\begin{code}
sem_return e1   =   return_ [e1]
syn_return      =   sem_return <$$ keyword "return" <**> syn_expr_lit <** keychar ';'
\end{code}%
\end{cofloat}%
%
Programs in which a procedure with $n$ formal parameters is invoked with $m \neq n$ actual parameters are invalid.
%
A procedure without a return command is fully executed, and returns no value as a result.
%
A procedure which is not guaranteed to return a value may only be invoked directly as a command, not as part of another expression.
%
If a procedure invocation raises an exception, the invocation is terminated and the exception is propagated.
%
The following example program produces no output:
%
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/no_output.mini}%
\end{cofloat}%

\section{Control Flow}
\label{sec:mini_control_flow}

\Mini{} has several control-flow constructs.
%
The basic constructs determine which code to execute based on Boolean conditions.

\paragraph{Basic Flow}

The \lstinline+if-then-else+-command is the \textbf{if} keyword followed by an r-value expression --- the condition --- and two code blocks, preceded by \textbf{then} and \textbf{else} respectively, of which the latter is optional.
%
\begin{cofloat}%
\begin{code}
sem_ite cond t e  = if_true_else_ [cond, t, e]
syn_ite           = sem_ite   <$$     keyword "if" <**> syn_expr_lit 
                              <**     keyword "then" <**> syn_branch 
                              <**>>>  optionalWithDef (keyword "else" **> syn_branch) null_
\end{code}%
\end{cofloat}%
%
To execute an \lstinline+if-then-else+-command, the condition is evaluated and, based on its value, one of the code blocks is executed.
%
An occurrence of \lstinline+if-then-else+ without an \textbf{else}-block is syntactic sugar for \lstinline+if-then-else+ with an empty \textbf{else}-block.
%

A \textbf{while}-loop is a command written as the \textbf{while} keyword followed by a condition and a code block.
%
A \textbf{while} command is executed by evaluating its condition and executing its body until the condition no longer holds.
%
The condition is checked before every iteration; if the condition is \lstinline-false-, the body is not executed at all.
%
\begin{cofloat}%
\begin{code}
sem_while cond body   = while_ [cond, body]
syn_while             = sem_while <$$ keyword "while" <**> syn_expr_lit <**> syn_branch
\end{code}%
\end{cofloat}%
%
A \textbf{for}-loop is a command written as the \textbf{for} keyword followed by three commands and a code block.
%
The commands must be an assignment --- the initialiser --- a Boolean r-value expression --- the condition --- and another assignment --- the updater.
%
A \textbf{for}-loop is syntactic sugar for a \textbf{while}-command in the conventional way: executing a \textbf{for}-loop is executing the initialiser, followed by a \textbf{while}-loop with the condition of the \textbf{for}-loop and a body which is the \textbf{for}-loop's body extended with the updater.
%
\begin{cofloat}%
\begin{code}
syn_for =  sem_for  <$$   keyword "for" <**>  
  syn_assign <**> syn_expr <** keychar ';' <**> syn_assign <**>  syn_branch  
sem_for init cond upd body = and_then init (sem_while cond (and_then body upd))
\end{code}%
\end{cofloat}%
%
The following programs are equivalent, printing the first 10 integers.
%
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/while.mini}
\lstinputlisting{../../examples/langs/mini/programs/for.mini}%
\end{cofloat}%

\paragraph{Exceptional Flow}
\label{sec:mini_exceptions}
%
\Mini{} has a basic mechanism for throwing and catching exceptions.
%
An exception is a string, thrown by the \textbf{throw}-command, which is written as the keyword \textbf{throw} followed by an r-value expression and a semicolon.
%
\begin{cofloat}%
\begin{code}
sem_throw str   = throw_ [string_ str]
syn_throw       = sem_throw <$$ keyword "throw" <**> string_lit <** keychar ';'
\end{code}%
\end{cofloat}%
%
Programs in which the expression of a \textbf{throw}-command does not evaluate to a string are invalid.
%
Throwing an exception terminates the execution of the surrounding code block, or of the program, if there is no surrounding code block.
%
An exception is propagated, eventually terminating the execution of the whole program, unless the exception occurs in the \textbf{try}-block of a \textbf{try-catch}-command, in which case it may be caught.
%

The \textbf{try-catch}-command is a series of code blocks, the first of which is the \textbf{try}-block, the others are \emph{handlers}.
%
Syntactically, the \textbf{try}-block is preceded by the \textbf{try} keyword, and the handlers are preceded by the \textbf{catch} keyword, followed by a string literal.
%
\begin{cofloat}%
\begin{code}
syn_trycatch =  sem_try <$$ keyword "try" <**> syn_branch <**> syn_handlers
sem_try tryblock handlers = handle_thrown_  [tryblock,handlers]

syn_handlers = "handlers" <:=> foldr_multiple syn_handler (throw_ [given_])
syn_handler :: SymbExpr Token (Funcons -> Funcons)
syn_handler = "single-handler" 
  <:=> sem_handler <$$ keyword "catch" <**> string_lit <**> syn_branch
sem_handler exc body handlers = 
  if_true_else_   [is_equal_ [string_ exc, given_], body, handlers]
\end{code}%
\end{cofloat}%
%
Executing a \textbf{try-catch}-command begins with executing its \textbf{try}-block.
%
The command has finished if the \textbf{try}-block is executed without throwing an exception.
%
If an exception $x$ is thrown, then the handlers are considered, in order, until the first handler that is applicable to $x$ is found.
%
A handler is applicable to $x$ if the string literal associated with the handler equals $x$.
%
When applicable, the handler is executed.
%
If executing a handler throws an exception, this exception is propagated.
%
If no handler is applicable to $x$, $x$ is propagated.
%

Not only user-thrown exceptions can be caught.
%
For example, the following program has handlers for |"uninitialised-reference-component"| and |"division-by-zero"| exceptions:
%
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/mynull.mini}%
\end{cofloat}%

\section{Arrays}
\label{sec:mini_arrays}

Besides integers, Booleans, and strings, \Mini{} also has arrays.
%
An array is an example of a \emph{composite variable}: a collection of variables with a particular structure.
%
An array has a fixed length $n$ and there is a variable associated with each index $0$ to $n-1$.
%
An array is an ordinary value, and can thus be assigned to a variable, returned by a procedure, etc.
%
To create arrays, \Mini{} has array-notation: a comma-separated sequence of r-value expressions surrounded by brackets.
%
\begin{cofloat}%
\begin{code}
syn_array = "array" <:=> 
  sem_array <$$> brackets (multipleSepBy syn_expr_lit (keychar ','))
sem_array elems = vector_ [left_to_right_ (map alloc elems)]
  where alloc e = allocate_initialised_variable_ [values_,e]
\end{code}%
\end{cofloat}%
%
The expressions are evaluated in left-to-right order, and each resulting value is assigned to a fresh variable.
%
For example, the expression \lstinline-[1,2,3,4,5]- creates an array consisting of five variables with indices 0 to 4, initialised with the numbers 1 to 5.
%

So far we have seen one example of an l-value expression: an expression containing just an identifier bound to a variable.
%
Array indexing is an example of another l-value expression.
%
An array indexing expression is written as an identifier and an r-value expression within brackets.
%
In valid programs, the identifier is bound to a variable with an array assigned to it, and the expression evaluates to an integer.
%
\begin{cofloat}%
\begin{code}
syn_array_idx = "array-index" <:=> sem_array_idx <$$> syn_id <**> brackets syn_expr_lit
\end{code}%
\end{cofloat}%
%
Evaluating an l-value expression $a[e]$ involves retrieving the array assigned to the variable to which $a$ is bound, and evaluating $e$ to $i$.
%
If $i$ is not an index of the array, an |"array-index-out-of-bounds"| exception is thrown.
%
If it is, the variable with index $i$ of the array is the result of evaluating the l-value expression.
%
\begin{cofloat}%
\begin{code}
sem_array_idx id e1 = give_   [e1, if_true_else_  [range, vector_index given_ vec
                              ,sem_throw "array-index-out-of-bounds"]]
  where vec     = ensure_lit (sem_var id)
        range   = and_  [is_greater_or_equal_ [given_, int_ 0]
                        ,is_less_ [given_, vector_length vec]]

vector_length v   = length_ [vector_elements_ [v]]
vector_index i v  = index_ [integer_add_ [int_ 1, i], vector_elements_ [v]]
\end{code}%
\end{cofloat}%
%
Because array indexing is an l-value expression, it can occur in places where a value is expected (see Section~\ref{sec:mini_variables}), as well as on the left-hand side of an assignment.
%
For example, the following program creates an array of length one and updates and accesses the variable it consists of:
%
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/array_basics.mini}%
\end{cofloat}%

\paragraph{Operations on arrays}

In Section~\ref{sec:mini_basic_exprs}, several built-in procedures were introduced.
%
The following additional built-in procedures are available for working with arrays:
%
\begin{cofloat}%
\begin{code}
array_built_ins = 
  [   ( "array_length", prefix_op [vector_length (assigned_ [given_])])
  ,   ( "empty_array", prefix_op [vector_ 
                      [interleave_repeat_ [allocate_variable_ [values_], int_ 1, assigned_ [given_]]]] ) ]
\end{code}%
\end{cofloat}%
%
The procedure \texttt{array\_length} is given an argument assigned to an array and returns the length of that array.
%
The procedure \texttt{empty\_array} receives as argument a variable assigned to $n$ and returns an array of length $n$, consisting of $n$ uninitialised variables.
%
The following programs shows how these procedures are used:
%
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/array_ops.mini}%
\end{cofloat}%


\section{Programs}
\label{sec:mini_programs}

A \Mini{} program is a sequence of declarations and commands executed in order.
%
\begin{cofloat}%
\begin{code}
syn_program = "program" <:=> sem_program <$$> syn_globals

sem_program body = 
  initialise_binding_ [initialise_storing_ [ 
    handle_thrown_ [scope_ [all_built_in_identifiers, body], handler]]]
  where handler = print_  [string_append_ 
                            [string_ "Uncaught Exception: "
                            ,given_,string_ "\n"]]

all_built_in_identifiers = env_fromlist_ $ array_built_ins ++ operator_semantics
\end{code}%
\end{cofloat}%
%
A program is executed as a code block (see Section~\ref{sec:mini_blocks}).
%
However, if a program terminates because an exception is thrown, the exception is reported.
%
For example, the message \verb+Uncaught Exception: division-by-zero+ is the output of the following program:
\begin{cofloat}%
\lstinputlisting{../../examples/langs/mini/programs/div0.mini}%
\end{cofloat}%
%

\section{Interpretation}
\label{sec:mini_interpretation}
%
This section discusses how the Haskell code segments from the previous sections form an interpreter for \Mini{}.
%
The interpreter is available as the \texttt{mini-reuse} package in the supplementary material.
%
The function |lexer :: String -> [Token]| is omitted.
%
The parser is defined as follows, applying |parse| from the \texttt{gll} package to |syn_program|:
%
\begin{cofloat}%
\begin{spec}
parser :: [Token] -> [Funcons]
parser = parse syn_program
\end{spec}%
\end{cofloat}%
%
Note that running the parser may result in several funcon terms, one for each possible interpretation of the input string.
%
Although several disambiguation strategies have been identified, e.g. longest-match to solve the `dangling else problem' (see Section~\ref{sec:mini_control_flow}), there is no guarantee that there is at most one interpretation for every program.
%
The |main| function defined below combines the lexer and parser to obtain zero or more funcon terms, and applies the function |runWithExtensions| imported from \texttt{Funcons.Tools} to execute them.
%
\begin{cofloat}%
\begin{code}
main = do
    args <- getArgs
    case args of 
      []        -> putStrLn "Please provide me with an input file"
      f:opts    -> go f opts
 where
    go :: FilePath -> [String] -> IO ()
    go f opts = do 
        str <- readFile f
        let   tokens      = lexer str
              fcts        = parser tokens
              ambiguous   = length fcts > 1
        forM_ (zip [1..] fcts) $ \(i, fct) -> do
            when ambiguous (putStrLn ("=== Interpretation "++ show i ++ "\n"))
            runWithExtensions Mini.funcons Mini.entities Mini.types opts (Just fct)
\end{code}%
\end{cofloat}%
%            writeFile (f ++ ".iml") (showResetBindings ++ "\n" ++ showQuery (mkQueryFromFCT fct ["standard-out", "store"]))

%
The first command line argument is expected to be a \Mini{} program.
%
The other command line arguments are passed on to |runWithExtensions| from the Haskell Funcon Framework.
%
The function |runWithExtensions| runs the funcon term interpreter on a given funcon term, extending the interpreter with the implementations of additional, language-specific funcons.
%
\label{sec:mini_show}
The only \Mini{}-specific funcon \cbsFC{mini-show} is defined below, and its implementation is exported by the module \texttt{Funcons.Mini.Mini}, imported under the qualified name |Mini|.
%
\begin{cbs*}
\cbsKeyword{Funcon} \;
  \cbsFC{mini-show}(\cbsParam{\cbsFCVar{V}}\cbsFC{values})\\&
\cbsKeyword{Rule}\;
    \cbsFC{mini-show}(\cbsFC{true}) \rarrow |"true"|\\&
\cbsKeyword{Rule}\;
    \cbsFC{mini-show}(\cbsFC{false}) \rarrow |"false"|\\&
\cbsKeyword{Rule}\;
    \cbsFC{mini-show}(\cbsParam{\cbsFCVar{I}}{\cbsFC{integers}}) \rarrow \cbsFC{to-string}(\cbsFCVar{I})\\&
\cbsKeyword{Rule}\;
    \cbsFC{mini-show}(\cbsParam{\cbsFCVar{S}}{\cbsFC{strings}}) \rarrow \cbsFCVar{S} \\&
\cbsKeyword{Rule}\;
    \cbsFC{mini-show}(\cbsFC{vectors}(\cbsStarFCVar{V})) \rarrow \\
    \cbsindent{3}\cbsFC{string-append}(|"["|, \cbsFC{intersperse}(|", "|, \\
    \cbsindent{4}\cbsFC{interleave-map}(\cbsFC{mini-show}(\cbsFC{assigned}(\cbsFC{given})),\cbsStarFCVar{V})), |"]"|)
\end{cbs*}

%if false
\begin{code}
parser :: [Token] -> [Funcons]
parser = parseWithParseOptions [] [maximumErrors 1, throwErrors] syn_program 
\end{code}
%endif 


